import { describe, beforeEach, it } from 'mocha'
import { assert } from 'chai'

import Dathost from '../index'
import Server from '../server'
import ServerSettings from '../settings/server'
import { IServer } from '../interfaces/server'
import Backup from '../server/backup'
import File from '../server/file'
import MatchSettings from '../settings/match'
import Match from '../match'


const generatePassword = (): string => {
  return Math.random().toString(24).substr(2, 16)
}


describe('dathost', () => {
    let dathost: Dathost

    beforeEach(() => {
        dathost = new Dathost(
            process.env.npm_package_config_datHostEmail || '',
            process.env.npm_package_config_datHostPass || ''
        )
    })

    it('Get account info', async () => {
        assert(await dathost.account() instanceof Object)
    })

    it('Get domains', async () => {
        for await (const domain of dathost.domains()) {
            assert(typeof domain === 'string')
        }
    })

    it('Get servers', async () => {
        for await (const server of dathost.servers()) {
            assert(server[0] instanceof Object)
            assert(server[1] instanceof Server)
        }
    })

    describe('CS: GO server', () => {
        let server: [IServer, Server]

        it('Create server', async () => {
            server = await dathost.createServer(new ServerSettings({
                name: 'TS CS: GO Server',
                location: 'sydney'
            }).csgo({
                slots: 5,
                gameToken: '',
                tickrate: 128,
                rconPassword: generatePassword()
            }))

            assert(server[0] instanceof Object)
            assert(server[1] instanceof Server)
        })
      
        it('Get console auth', async () => {
            assert(await server[1].consoleAuth() instanceof Object)
        })

        it('Get server details', async () => {
            assert(await server[1].get() instanceof Object)
        })

        it('Get server metrics', async () => {
            assert(await server[1].metrics() instanceof Object)
        })

        it('Update server', async () => {
            await server[1].update(new ServerSettings({
                name: 'TS CS: GO Server update'
            }))
        })

        it('Start server', async () => {
            await server[1].start()
        })

        it('Stop server', async () => {
            await server[1].stop()
        })

        it('Reset server', async () => {
            await server[1].reset()
        })

        it('Regenerate ftp password', async () => {
            await server[1].regenerateFtpPassword()
        })

        it('Sync files', async () => {
            await server[1].syncFiles()
        })

        it('Console retrieve', async () => {
            await server[1].consoleRetrieve()
        })

        it('Console send', async () => {
            await server[1].consoleSend('say https://github.com/UnofficialDathost/TypeScript')
        })

        it('Duplicate server', async () => {
            const serverDup: [IServer, Server] = await server[1].duplicate()
            assert(serverDup[0] instanceof Object)
            assert(serverDup[1] instanceof Server)
            await serverDup[1].delete()
        })

        it('List backups on servers', async () => {
            for await (const backup of server[1].backups()) {
                assert(backup[0] instanceof Object)
                assert(backup[1] instanceof Backup)
                await backup[1].restore()
            }
        })

        it('List files on server', async () => {
            for await (const file of server[1].files()) {
                assert(file[0] instanceof Object)
                assert(file[1] instanceof File)
            }
        })

        it('Delete server', async () => {
            await server[1].delete()
        })

        it('Create match', async () => {
            const matchServer = await dathost.createServer(new ServerSettings({
              name: 'TS CS:GO Match',
              location: 'sydney'
            }).csgo({
              slots: 5,
              gameToken: '',
              tickrate: 128,
              rconPassword: generatePassword()
            }))

            const match = await matchServer[1].createMatch(new MatchSettings({
                connectionTime: 300,
                knifeRound: false,
                waitForSpectators: false,
                warmupTime: 15
            }).team_1(
                [
                    "[U:1:116962485]",
                    76561198017567105,
                    "STEAM_0:1:186064092",
                    "76561198214871321"
                ]
            ).team_2(
                [
                    "[U:1:320762620]",
                    "STEAM_1:1:83437164",
                    76561198214871324,
                    "76561198214871323"
                ]
            ))

            assert(match[0] instanceof Object)
            assert(match[1] instanceof Match)

            assert(await match[1].get() instanceof Object)
      })
    })

    describe('TF2 Server', () => {
        let server: [IServer, Server]

        it('Create server', async () => {
            server = await dathost.createServer(new ServerSettings({
                name: 'TS TF2 Server',
                location: 'sydney'
            }).tf2({
                slots: 5,
                rconPassword: generatePassword()
            }))
        })

        it('Get server details', async () => {
            assert(await server[1].get() instanceof Object)
        })

        it('Get server metrics', async () => {
            assert(await server[1].metrics() instanceof Object)
        })
    
        it('Start server', async () => {
            await server[1].start(true)
        })

        it('Update server', async () => {
            await server[1].update(new ServerSettings({
                name: 'TS TF2 Server update'
            }))
        })

        it('Stop server', async () => {
            await server[1].stop()
        })

        it('Reset server', async () => {
            await server[1].reset()
        })

        it('Regenerate ftp password', async () => {
            await server[1].regenerateFtpPassword()
        })

        it('Sync files', async () => {
            await server[1].syncFiles()
        })

        it('Console retrieve', async () => {
            await server[1].consoleRetrieve(300)
        })

        it('Console send', async () => {
            await server[1].consoleSend('say https://github.com/UnofficialDathost/TypeScript')
        })

        it('List files on server', async () => {
            for await (const file of server[1].files({hideDefaultFiles: true, deletedFiles: true, fileSizes: true})) {
                assert(file[0] instanceof Object)
                assert(file[1] instanceof File)
            }
        })

        it('Delete server', async () => {
            await server[1].delete()
        })
    })

    describe('Valheim server', () => {
        let server: [IServer, Server]

        it('Create server', async () => {
            server = await dathost.createServer(new ServerSettings({
                name: 'TS Valheim server',
                location: 'sydney'
            }).valheim({
                password: generatePassword(),
                worldName: 'dathost ts',
                plus: false,
                admins: ['[U:1:116962485]', 'STEAM_0:1:186064092',
                         '76561198017567105', 76561198214871321]
            }))
        })

        it('Get server details', async () => {
            assert(await server[1].get() instanceof Object)
        })

        it('Get server metrics', async () => {
            assert(await server[1].metrics() instanceof Object)
        })

        it('Update server', async () => {
            await server[1].update(new ServerSettings({
                name: 'TS Valheim Server update'
            }))
        })

        it('Start server', async () => {
            await server[1].start(false)
        })

        it('Stop server', async () => {
            await server[1].stop()
        })

        it('Reset server', async () => {
            await server[1].reset()
        })

        it('Regenerate ftp password', async () => {
            await server[1].regenerateFtpPassword()
        })

        it('Sync files', async () => {
            await server[1].syncFiles()
        })

        it('Console retrieve', async () => {
            await server[1].consoleRetrieve()
        })

        it('Console send', async () => {
            await server[1].consoleSend('say https://github.com/UnofficialDathost/TypeScript')
        })

        it('Delete server', async () => {
            await server[1].delete()
        })
    })

    describe('Teamspeak server', () => {
        let server: [IServer, Server]

        it('Create server', async () => {
            server = await dathost.createServer(new ServerSettings({
                name: 'TS Teamspeak server',
                location: 'sydney'
            }).teamspeak({slots: 5}))
        })

        it('Get server details', async () => {
            assert(await server[1].get() instanceof Object)
        })

        it('Get server metrics', async () => {
            assert(await server[1].metrics() instanceof Object)
        })

        it('Update server', async () => {
            await server[1].update(new ServerSettings({
                name: 'TS Teamspeak Server update'
            }))
        })

        it('Start server', async () => {
            await server[1].start()
        })

        it('Stop server', async () => {
            await server[1].stop()
        })

        it('Reset server', async () => {
            await server[1].reset()
        })

        it('Regenerate ftp password', async () => {
            await server[1].regenerateFtpPassword()
        })

        it('Sync files', async () => {
            await server[1].syncFiles()
        })

        it('Delete server', async () => {
            await server[1].delete()
        })
    })
})
